;/*
sc link optimize debug=line idir=include: nostackcheck opt data=far nostartup Touch.c
quit
*/

#define USE_SOFTLINKCHECK		1

#include <string.h>

#include <exec/exec.h>
#include <dos/dos.h>
#include <dos/exall.h>
#include <dos/dosasl.h>

#include <proto/exec.h>
#include <proto/dos.h>

#ifdef __SASC
typedef LONG IPTR;
#endif


#define	DEBUG(x)	;


#include "touch_version.h"

static const char version[] = VERSTAG;

#define	ARG_NAME	0
#define	ARG_VERBOSE	1
#define	ARG_ALL		2
#define	ARG_COUNT	3

static
BOOL touchentry(const UBYTE *dirname, int Verbose, struct DosLibrary *DOSBase);

static
BOOL touchentry_ap(struct AnchorPath *ap, int Verbose, struct DosLibrary *DOSBase);

int main(void)
{
	struct ExecBase     *SysBase;
	struct DosLibrary   *DOSBase;
	const UBYTE         *Header = NULL;
	LONG                ret = RETURN_FAIL;
	ULONG               MatchFlag = FALSE;

	SysBase = *((struct ExecBase **) 4);
	DOSBase = (struct DosLibrary *) OpenLibrary("dos.library", 37);
	if (DOSBase)
	{
		struct RDArgs  *rda;
		IPTR           args[ARG_COUNT];
		LONG           err;

		ret = RETURN_ERROR;

		memclr(&args, sizeof(args));
		rda = ReadArgs("NAME/A/M,VERBOSE/S,ALL/S", args, NULL);
		if (rda)
		{
			UBYTE               stack_ap[sizeof(struct AnchorPath) + 3];
			struct AnchorPath   *MyAp = (struct AnchorPath *) (((ULONG) stack_ap + 3) & ~3);
			UBYTE               dirname[1024];
			const UBYTE * const *MyName;

			MyName =(const UBYTE * const *) args[ARG_NAME];
			ret = RETURN_OK;

			while (*MyName)
			{
				DEBUG(Printf("touch: handle <%s>\n",
				             *MyName));

				MatchFlag	=	TRUE;

				memclr(MyAp, sizeof(*MyAp));

				for (err = MatchFirst(*MyName, MyAp);
				     !err;
				     err = MatchNext(MyAp))
				{
					DEBUG({
					if (!NameFromLock(MyAp->ap_Current->an_Lock,
					                  dirname,
					                  sizeof(dirname)))
					{
						PrintFault(IoErr(),
						           "Error on NameFromLock");
						break;
					}
					if (!AddPart(dirname,
					             MyAp->ap_Info.fib_FileName,
					             sizeof(dirname)))
					{
						PrintFault(IoErr(),
						           "Error on AddPart");
						break;
					}
					});

					DEBUG(Printf("touch: path <%s>\n",
					             dirname));

					/* This code section deals with ALL and directory ascent/descent */

#if USE_SOFTLINKCHECK
					if (SoftlinkDODIR(MyAp, TRUE, TRUE, DOSBase))
#else /* USE_SOFTLINKCHECK */
					if (MyAp->ap_Info.fib_DirEntryType > 0)
#endif /* USE_SOFTLINKCHECK */
					{
						if (MyAp->ap_Flags & APF_DIDDIR)
						{
							DEBUG(Printf("Touch: Ascending from directory %s\n",
							             dirname));

							touchentry_ap(MyAp, args[ARG_VERBOSE], DOSBase);
						}
#if 1
						else if (args[ARG_ALL])
						{
							DEBUG(Printf("Touch: Entering directory %s\n",
								             dirname));
							/* make it enter the directory */
							MyAp->ap_Flags   |=      APF_DODIR;
						}
#endif /* 1 */
						else
						{
							DEBUG(Printf("Touch: The next dir is  ... %s\n",
							             dirname));

							touchentry_ap(MyAp, args[ARG_VERBOSE], DOSBase);
						}

						/* clear the completed directory flag */
						MyAp->ap_Flags     &=      ~APF_DIDDIR;
					}
					else
					{
						LONG res;

						res = touchentry_ap(MyAp, args[ARG_VERBOSE], DOSBase);

						if (!res)
						{
							err = IoErr();
							if (NameFromLock(MyAp->ap_Current->an_Lock,
							                 dirname,
							                 sizeof(dirname)) &&
							    AddPart(dirname,
							            MyAp->ap_Info.fib_FileName,
							            sizeof(dirname)))
							{
								Header = dirname;
							}

							ret = RETURN_FAIL;
							goto error;
						}
					}

					if (SetSignal(0L,SIGBREAKF_CTRL_C) & SIGBREAKF_CTRL_C)
					{
						SetIoErr(ERROR_BREAK);
						goto error;
					}
				}

				err = IoErr();

				MatchEnd(MyAp);
				MatchFlag	=	FALSE;

				if (err == ERROR_OBJECT_NOT_FOUND)
				{
					DEBUG(Printf("touch: file <%s> didn't match pattern, touch directly\n", *MyName));
					if (!touchentry(*MyName, args[ARG_VERBOSE], DOSBase))
					{
						DEBUG(Printf("Touch: failed\n"));
						Header = *MyName;
						ret = RETURN_FAIL;
						err = IoErr();
						goto error;
					}
					else
					{
						err = IoErr();
					}
				}
				else if (err != ERROR_NO_MORE_ENTRIES)
				{
					DEBUG(Printf("Touch: error %ld\n", err));
					goto error;
				}

				MyName++;
			}
error:
			if (MatchFlag)
			{
				MatchEnd(MyAp);
			}

			// Don't FreeArgs before PrintFault, Header might point to arg array!
			//FreeArgs(rda);
		}
		else
		{
			err = IoErr();
		}

		if (ret != RETURN_OK &&
		    err != ERROR_NO_MORE_ENTRIES)
		{
			PrintFault(err, Header);
		}

		if (rda)
			FreeArgs(rda);

		if (err == ERROR_NO_MORE_ENTRIES)
			err = 0;

		SetIoErr(err);

		CloseLibrary((struct Library *) DOSBase);
	}

	return ret;
}


static
BOOL touchentry(const UBYTE *dirname, int Verbose, struct DosLibrary *DOSBase)
{
	struct DateStamp	MyDateStamp;
	BPTR			fh;
	BOOL			ret = TRUE;

	DEBUG(Printf("touch: touch file <%s>\n", dirname));
	/* Here is code for handling each particular file */
	if (Verbose)
	{
		Printf("%s...",
			(IPTR) dirname);
	}
#if 1
	DateStamp(&MyDateStamp);
	if (SetFileDate(dirname,
	                &MyDateStamp))
	{
		DEBUG(Printf("touchentry: setfiledate worked\n"));
		if (Verbose)
		{
			Printf("touched\n");
		}
	}
	else if (IoErr() == ERROR_OBJECT_NOT_FOUND)
#endif
	{
		if ((fh = Open(dirname, MODE_READWRITE)))
		{
			if (Verbose)
			{
				Printf("created\n");
			}
			DEBUG(Printf("touchentry: create file worked\n"));
			Close(fh);
		}
		else
		{
			if (Verbose)
			{
				Printf("failed\n");
			}
			ret = FALSE;
		}
	}

	DEBUG(Printf("touchentry: ret %ld\n",ret));

	return ret;
}

static
BOOL touchentry_ap(struct AnchorPath *ap, int Verbose, struct DosLibrary *DOSBase)
{
	BPTR olddir;
	BOOL ret;

	olddir = CurrentDir(ap->ap_Current->an_Lock);

	ret = touchentry(ap->ap_Info.fib_FileName, Verbose, DOSBase);

	CurrentDir(olddir);

	return ret;
}

